'use strict';
var util = require('util');
var path = require('path');
var yeoman = require('yeoman-generator');
// Extra requires
var fs = require('fs-extra');
var passthru = require('passthru');


var KwebappGenerator = module.exports = function KwebappGenerator(args, options, config) {
    yeoman.generators.Base.apply(this, arguments);

    this.on('end', function () {
        this.installDependencies({skipInstall: options['skip-install']});
    });

    this.pkg = JSON.parse(this.readFileAsString(path.join(__dirname, '../package.json')));

};

util.inherits(KwebappGenerator, yeoman.generators.Base);

function _camelize(str) {
    return str.replace(/(?:^\w|[A-Z]|\b\w)/g, function (letter, index) {
        return index === 0 ? letter.toLowerCase() : letter.toUpperCase();
    }).replace(/\s+/g, '');
}

function _randomString(len, charSet) {
    charSet = charSet || 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    var randomString = '';
    for (var i = 0; i < len; i++) {
        var randomPoz = Math.floor(Math.random() * charSet.length);
        randomString += charSet.substring(randomPoz, randomPoz + 1);
    }
    return randomString;
}

function _humanize(str) {
    return str.replace(/_/g, ' ')
            .replace(/-/g, ' ')
            .replace(/(\w+)/g, function (match) {
                return match.charAt(0).toUpperCase() + match.slice(1);
            });
}

function _slugify(str) {
    return str.toLowerCase().replace(/-+/g, '').replace(/\s+/g, '-').replace(/[^a-z0-9-]/g, '');
}

KwebappGenerator.prototype.askFor = function askFor() {
    var cb = this.async();

    // have Yeoman greet the user.
    console.log(this.yeoman);

    var prompts = [
        {
            name: 'projectName',
            message: 'What\'s the project name?',
            default: 'Web Application'
        },
        {
            name: 'projectBasepath',
            message: 'What\'s the base path of this project?',
            default: '/'
        },
        {
            type: 'checkbox',
            name: 'mainDevelopers',
            message: 'Which developers will work in this project?',
            choices: [
                "Alexandre Moraes <alexandrecmoraes89@gmail.com>"
            ]
        },
        {
            type: 'confirm',
            name: 'allowExtraDev',
            message: 'Want to add extra developers?',
            default: false
        },
        {
            when: function (props) {
                if (props.allowExtraDev) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'extraDevelopers',
            message: 'Specify the extra developers to this project separated by comma',
            default: 'Abdullah <abdul@gmail.com>'
        },
        {
            type: 'confirm',
            name: 'installCake',
            message: 'Would you like to install CakePHP?',
            default: false
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'cakeVersion',
            message: 'Which version of CakePHP should I install?',
            default: '2.5.6'
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'environmentName',
            message: 'What\s the name of this host?',
            default: 'dev'
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'databaseType',
            message: 'Database type',
            default: 'Database/Mysql'
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'databaseName',
            message: 'Database name',
            default: 'webapp'
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'databaseHost',
            message: 'Database host',
            default: 'localhost'
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'databaseUsername',
            message: 'Database username',
            default: 'root'
        },
        {
            when: function (props) {
                if (props.installCake) {
                    return true;
                } else {
                    return false;
                }
            },
            type: 'password',
            name: 'databasePassword',
            message: 'Database password',
            default: ''
        },
        {
            type: 'checkbox',
            name: 'projectAdditional',
            message: 'Which functionalities do you want',
            choices: ["AngularJS", "Bootstrap", "Bootflat", "jQuery", "RequireJS", "FontAwesome"]
        },
        {
            when: function (props) {
                if (props.projectAdditional.indexOf('AngularJS') > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'angularjsVersion',
            message: 'Which version of AngularJS should I install?',
            default: '1.3.3'
        },
        {
            when: function (props) {
                if (props.projectAdditional.indexOf('Bootstrap') > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'bootstrapVersion',
            message: 'Which version of Bootstrap should I install?',
            default: '~3.1.1'
        },
        {
            when: function (props) {
                if (props.projectAdditional.indexOf('Bootflat') > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'bootflatVersion',
            message: 'Which version of Bootflat should I install?',
            default: 'master'
        },
        {
            when: function (props) {
                if (props.projectAdditional.indexOf('jQuery') > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'jqueryVersion',
            message: 'Which version of jQuery should I install?',
            default: '1.11.0'
        },
        {
            when: function (props) {
                if (props.projectAdditional.indexOf('RequireJS') > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'requirejsVersion',
            message: 'Which version of RequireJS should I install?',
            default: '~2.1.15'
        },
        {
            when: function (props) {
                if (props.projectAdditional.indexOf('FontAwesome') > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            name: 'fontawesomeVersion',
            message: 'Which version of FontAwesome should I install?',
            default: '~4.1.0'
        }
    ];

    this.prompt(prompts, function (props) {
        this.projectName = props.projectName;
        this.projectBasepath = props.projectBasepath;
        this.mainDevelopers = props.mainDevelopers;
        this.camelizedProjectName = _camelize(this.projectName);
        this.installCake = props.installCake;
        if (props.allowExtraDev) {
            this.mainDevelopers = this.mainDevelopers.concat(props.extraDevelopers);
        }
        if (this.installCake) {
            this.webrootPath = 'app/webroot/';

            this.cakeVersion = props.cakeVersion;
            this.environmentName = props.environmentName;
            this.databaseType = props.databaseType;
            this.databaseHost = props.databaseHost;
            this.databaseName = props.databaseName;
            this.databaseUsername = props.databaseUsername;
            this.databasePassword = props.databasePassword;
        } else {
            this.webrootPath = 'assets/';
        }

        this.projectAdditional = props.projectAdditional;

        if (this.projectAdditional.indexOf('AngularJS') > -1) {
            this.angularjsVersion = props.angularjsVersion;
        }

        if (this.projectAdditional.indexOf('Bootstrap') > -1) {
            this.bootstrapVersion = props.bootstrapVersion;
        }

        if (this.projectAdditional.indexOf('jQuery') > -1) {
            this.jqueryVersion = props.jqueryVersion;
        }

        if (this.projectAdditional.indexOf('RequireJS') > -1) {
            this.requirejsVersion = props.requirejsVersion;
        }

        if (this.projectAdditional.indexOf('FontAwesome') > -1) {
            this.fontawesomeVersion = props.fontawesomeVersion;
        }

        if (this.projectAdditional.indexOf('Bootflat') > -1) {
            this.bootflatVersion = props.bootflatVersion;
        }

        cb();
    }.bind(this));
};

KwebappGenerator.prototype.createCommon = function createCommon() {
    var cb = this.async();

    // Create common folders and files for test
    this.mkdir('test');
    this.mkdir('test/lib');

    this.template('_webfiles/_interface.test.html', 'test/interface.js/interface.test.html');
    this.template('_webfiles/_util.test.html', 'test/util.js/util.test.html');
    this.copy('_webfiles/_interface.test.js', 'test/interface.js/interface.test.js');
    this.copy('_webfiles/_util.test.js', 'test/util.js/util.test.js');

    if (!this.installCake) {
        // Create index.html if CakePHP has not been choosen
        if (this.projectAdditional.indexOf('AngularJS') > -1) {
            this.template('_webfiles/indexes/_angular.html', 'index.html');
        } else {
            this.template('_webfiles/indexes/_index.html', 'index.html');
        }
    }

    // Create other custom files and folders
    this.mkdir(this.webrootPath);
    this.mkdir(this.webrootPath + 'img');
    this.mkdir(this.webrootPath + 'includes');
    this.mkdir(this.webrootPath + 'css');
    this.template('_webfiles/_style.css', this.webrootPath + 'css/style.css');
    this.mkdir(this.webrootPath + 'js');

    this.template('_webfiles/_interface.js', this.webrootPath + 'js/interface.js');
    this.copy('_webfiles/_util.js', this.webrootPath + 'js/util.js');

    // Create components folder and copy components if it have
    this.mkdir(this.webrootPath + 'vendors');

//    if (this.projectAdditional.indexOf('jQuery') > -1) {
//        this.mkdir(this.webrootPath + 'com/js/jquery');
//    }

    // If Angular is installed...
    if (this.projectAdditional.indexOf('AngularJS') > -1) {
        this.mkdir(this.webrootPath + 'js/app');

        this.requireBaseUrl = this.projectBasepath + (this.installCake ? 'app/webroot/' : 'assets/');

        this.template('_webfiles/_mainAngular/_main.js', this.webrootPath + 'js/app/main/main.js');
        this.copy('_webfiles/_mainAngular/_mainApp.js', this.webrootPath + 'js/app/main/mainApp.js');
        this.copy('_webfiles/_mainAngular/_mainControllers.js', this.webrootPath + 'js/app/main/mainControllers.js');
        this.copy('_webfiles/_mainAngular/_mainServices.js', this.webrootPath + 'js/app/main/mainServices.js');
    }

//    if (this.projectAdditional.indexOf('Bootstrap') > -1) {
//        this.mkdir(this.webrootPath + 'com/js/bootstrap');
//        this.mkdir(this.webrootPath + 'com/css/bootstrap');
//    }

    cb();
};

KwebappGenerator.prototype.packageCopy = function packageCopy() {
    var cb = this.async();
    this.template('_package.json', 'package.json');
    cb();
};

KwebappGenerator.prototype.updatePackage = function updatePackage() {

    var cb = this.async();

    var appPkg = JSON.parse(this.readFileAsString('package.json'));

    // Add the mainDevelopers list to the Contributors List
    for (var contrib in this.mainDevelopers) {
        var current = {
            "name": this.mainDevelopers[contrib]
        };
        appPkg.contributors.push(current);
    }

    // Add the webrootPath as property
    appPkg.webrootPath = this.webrootPath;

    // Write the modified package.json to the file
    fs.writeFile('package.json', JSON.stringify(appPkg, null, 4), 'utf8', function (err) {
        if (err)
            return console.log(err);
        cb();
    });
};

KwebappGenerator.prototype.projectfiles = function projectfiles() {
    var cb = this.async();
    if (this.installCake) {
        this.template('_composer.json', 'composer.json');
    }

    this.template('_bower.json', 'bower.json');
    this.template('_bowerrc', '.bowerrc');

    this.copy('_Gruntfile.js', 'Gruntfile.js');
    this.copy('editorconfig', '.editorconfig');
    this.copy('jshintrc', '.jshintrc');

    cb();
};

KwebappGenerator.prototype.composer = function composer() {
    var cb = this.async();
    // Download CakePHP if chosen
    if (this.installCake) {
        this.fetch('https://getcomposer.org/composer.phar', 'composer', function (err) {
            if (err)
                return done(err);
            passthru("php composer/composer.phar install", function () {
                fs.copy('vendors/cakephp/cakephp/app/', 'app', function () {
                    fs.copy('vendors/cakephp/cakephp/lib/', 'lib', function () {
                        cb();
                    });
                });
            });
        });
    } else {
        cb();
    }
};

KwebappGenerator.prototype.setAuthors = function setAuthors() {
    var cb = this.async();

    // Set the specified authors to JSDoc into Style and Interface static files
    var mainDev = "";
    for (var dev in this.mainDevelopers) {
        mainDev = mainDev + "\n * @author " + this.mainDevelopers[dev];
    }
    var webrootPath = this.webrootPath;
    // Write to style.css
    fs.readFile(webrootPath + 'css/style.css', 'utf8', function (err, data) {
        if (err)
            return console.log(err);

        var result = data.replace('{mainDevelopers}', mainDev);

        fs.writeFile(webrootPath + 'css/style.css', result, 'utf8', function (err) {
            if (err)
                return console.log(err);
        });
    });
    // Write to interface.js
    fs.readFile(webrootPath + 'js/interface.js', 'utf8', function (err, data) {
        if (err)
            return console.log(err);

        var result = data.replace('{mainDevelopers}', mainDev);

        fs.writeFile(webrootPath + 'js/interface.js', result, 'utf8', function (err) {
            if (err)
                return console.log(err);
        });
        cb();
    });
};

KwebappGenerator.prototype.downloadDependencies = function downloadDependencies() {

    // SENT TO BOWER

//    Download jQuery
//    if (this.projectAdditional.indexOf('jQuery') > -1) {
//        this.fetch('http://code.jquery.com/jquery-' + this.jqueryVersion + '.min.js', this.webrootPath + 'com/js/jquery', function(err) {
//            if (err)
//                return done(err);
//        });
//        this.fetch('http://code.jquery.com/jquery-' + this.jqueryVersion + '.min.map', this.webrootPath + 'com/js/jquery', function(err) {
//            if (err)
//                return done(err);
//        });
//    }
//
//    // Download Bootstrap
//    if (this.projectAdditional.indexOf('Bootstrap') > -1) {
//        this.fetch('http://getbootstrap.com/dist/css/bootstrap.min.css', this.webrootPath + 'com/css/bootstrap', function(err) {
//            if (err)
//                return done(err);
//        });
//        this.fetch('http://getbootstrap.com/dist/js/bootstrap.min.js', this.webrootPath + 'com/js/bootstrap', function(err) {
//            if (err)
//                return done(err);
//        });
//    }

    // Install qunit test lib
    this.fetch('http://code.jquery.com/jquery-1.7.2.min.js', 'test/lib', function (err) {
        if (err)
            return done(err);
    });
    this.fetch('http://code.jquery.com/qunit/qunit-1.14.0.js', 'test/lib', function (err) {
        if (err)
            return done(err);
    });
    this.fetch('http://code.jquery.com/qunit/qunit-1.14.0.css', 'test/lib', function (err) {
        if (err)
            return done(err);
    });

};

KwebappGenerator.prototype.cakesetup = function cakesetup() {
    var cb = this.async();
    if (this.installCake) {
        // Installs the environment configuration files
        this.mkdir('app/Config/Environment');
        this.template('_cakefiles/Environment/_development.php', 'app/Config/Environment/' + this.environmentName + '.php');
        this.template('_cakefiles/Environment/_default.php', 'app/Config/Environment/default.php');

        // Recreate the AppController and AppModel files
        fs.remove('app/Controller/AppController.php');
        fs.remove('app/Model/AppModel.php');
        this.copy('_cakefiles/_AppController.php', 'app/Controller/AppController.php');
        this.copy('_cakefiles/_AppModel.php', 'app/Model/AppModel.php');

        // Additional custom files 
        this.copy('_cakefiles/_autoloader.ini', 'app/Vendor/cakephp/autoloader.ini');
        this.copy('_cakefiles/_database.php', 'app/Config/database.php');
        this.copy('_cakefiles/_index.php', 'index.php');
        this.copy('_cakefiles/_htaccess', '.htaccess');

        // Add Bootstrap welcome screen
        fs.remove('app/View/Layouts/default.ctp');
        fs.remove('app/View/Pages/home.ctp');

        if (this.projectAdditional.indexOf('AngularJS') > -1) {
            this.template('_cakefiles/indexes/_angular.ctp', 'app/View/Layouts/default.ctp');
        } else {
            this.template('_cakefiles/indexes/_default.ctp', 'app/View/Layouts/default.ctp');
        }
        this.template('_cakefiles/_home.ctp', 'app/View/Pages/home.ctp');

        // Set random values to Salt and Cipherseed
        fs.readFile('app/Config/core.php', 'utf8', function (err, data) {
            if (err)
                return console.log(err);

            var result = data.replace('DYhG93b0qyJfIxfs2guVoUubWwvniR2G0FgaC9mi', _randomString(40));
            result = result.replace('76859309657453542496749683645', _randomString(30, '0123456789'));

            fs.writeFile('app/Config/core.php', result, 'utf8', function (err) {
                if (err)
                    return console.log(err);
            });
        });

        // Set Bootstrap to load environment based config, load plugins and fix some issues
        fs.appendFile('app/Config/bootstrap.php', [
            "// Bootstrap for the different environments",
            "if(is_file(APP . 'Config' . DS . 'Environment' . DS . php_uname(\'n\').'.php')) {",
            "  Configure::load('Environment/'.php_uname('n'));",
            "} else {",
            "  Configure::load('Environment/default');",
            "}",
            "",
            "CakePlugin::loadAll();",
            "",
            "// Remove and re-prepend CakePHP's autoloader as composer thinks it is the most important.",
            "// See https://github.com/composer/composer/commit/c80cb76b9b5082ecc3e5b53b1050f76bb27b127b",
            "spl_autoload_unregister(array('App', 'load'));",
            "spl_autoload_register(array('App', 'load'), true, true);"
        ].join('\n'), 'utf8', function (err) {
            if (err)
                return console.log(err);
        });
        cb();
    } else {
        cb();
    }
};

KwebappGenerator.prototype.setTemplates = function setTemplates() {
    var cb = this.async();

    var basePath = this.projectBasepath;

    // Set the style and js tags into 'index.html' if user selected Bootstrap/jQuery/Angular/Require

    var hasAngular = this.projectAdditional.indexOf('AngularJS');

    var hasRequirejs = this.projectAdditional.indexOf('RequireJS');

    if (hasRequirejs > -1) {
        var requireTag = this.installCake ? "echo $this->Html->script('../vendors/requirejs/require'); " : "<script src=\"" + basePath + "assets/vendors/requirejs/require.js\"></script> ";
    }

    var hasJquery = this.projectAdditional.indexOf('jQuery');

    if (hasJquery > -1) {
        var jqueryTag = this.installCake ? "echo $this->Html->script('../vendors/jquery/dist/jquery.min'); " : "<script src=\"" + basePath + "assets/vendors/jquery/dist/jquery.min.js\"></script> ";
    }

    var hasBootstrap = this.projectAdditional.indexOf('Bootstrap');

    if (hasBootstrap > -1) {
        var bootstrapCssTag = this.installCake ? "echo $this->Html->css('../vendors/bootstrap/dist/css/bootstrap.min'); " : "<link type=\"text/css\" rel=\"stylesheet\" href=\"" + basePath + "assets/vendors/bootstrap/dist/css/bootstrap.min.css\" /> ";
        var bootstrapJsTag = this.installCake ? "echo $this->Html->script('../vendors/bootstrap/dist/js/bootstrap.min'); " : "<script src=\"" + basePath + "assets/vendors/bootstrap/dist/js/bootstrap.min.js\"></script> ";
    }

    var hasBootflat = this.projectAdditional.indexOf('Bootflat');

    if (hasBootflat > -1) {
        var bootflatCssTag = this.installCake ? "echo $this->Html->css('../vendors/bootflat/css/bootflat'); " : "<link type=\"text/css\" rel=\"stylesheet\" href=\"" + basePath + "assets/vendors/bootflat/css/bootflat.css\" /> ";
    }

    var hasFontAwesome = this.projectAdditional.indexOf('FontAwesome');

    if (hasFontAwesome > -1) {
        var fontAwesomeCssTag = this.installCake ? "echo $this->Html->css('../vendors/fontawesome/css/font-awesome.min'); " : "<link type=\"text/css\" rel=\"stylesheet\" href=\"" + basePath + "assets/vendors/fontawesome/css/font-awesome.min.css\" /> ";
    }

    var layoutPath = this.installCake ? 'app/View/Layouts/default.ctp' : 'index.html';

    fs.readFile(layoutPath, 'utf8', function (err, data) {
        if (err)
            return console.log(err);

        var cssData = hasBootstrap > -1 ? bootstrapCssTag : "";
        cssData = hasFontAwesome > -1 ? cssData + fontAwesomeCssTag : cssData;
        cssData = hasBootflat > -1 ? cssData + bootflatCssTag : cssData;

        var jsData = hasRequirejs > -1 ? requireTag : "";
        jsData = hasJquery > -1 ? jsData + jqueryTag : jsData;
        jsData = hasBootstrap > -1 ? jsData + bootstrapJsTag : jsData;

        if (hasAngular > -1) {
            jsData = layoutPath.indexOf('ctp') > -1 ? jsData + "echo $this->Html->script(array('app/main/main')); " : jsData + "<script src=\"" + basePath + "assets/js/app/main/main.js\"></script> ";
        }

        var result = data.replace('{includeCss}', cssData);
        result = result.replace('{includeJs}', jsData);

        fs.writeFile(layoutPath, result, 'utf8', function (err) {
            if (err)
                return console.log(err);
            cb();
        });
    });
};